@@ -19,8 +19,8 @@
  */
 
 #include "notifybypopup.h"
-#include "knotifyconfig.h"
 #include "imageconverter.h"
+#include "notifybypopupgrowl.h"
 
 #include <kdebug.h>
 #include <kpassivepopup.h>
@@ -29,6 +29,7 @@
 #include <khbox.h>
 #include <kvbox.h>
 #include <kcharsets.h>
+#include <knotifyconfig.h>
 
 #include <QBuffer>
 #include <QImage>
@@ -96,6 +97,28 @@ void NotifyByPopup::notify( int id, KNotifyConfig * config )
 		return;
 	}
 
+	// Default to 6 seconds if no timeout has been defined
+	int timeout = config->timeout == -1 ? 6000 : config->timeout;
+
+	// if Growl can display our popups, use that instead
+	if(NotifyByPopupGrowl::canPopup())
+	{
+		KNotifyConfig *c = ensurePopupCompatibility( config );
+
+		QString appCaption, iconName;
+		getAppCaptionAndIconName(c, &appCaption, &iconName);
+
+		KIconLoader iconLoader(iconName);
+		QPixmap appIcon = iconLoader.loadIcon( iconName, KIconLoader::Small );
+
+		NotifyByPopupGrowl::popup( &appIcon, timeout, appCaption, c->text );
+
+		// Finish immediately, because current NotifyByPopupGrowl can't callback
+		finish(id);
+		delete c;
+		return;
+	}
+
 	KPassivePopup *pop = new KPassivePopup( config->winId );
 	m_popups[id]=pop;
 	fillPopup(pop,id,config);
@@ -103,10 +126,7 @@ void NotifyByPopup::notify( int id, KNotifyConfig * config )
 	pop->setAutoDelete( true );
 	connect(pop, SIGNAL(destroyed()) , this, SLOT(slotPopupDestroyed()) );
 
-	// Default to 6 seconds if no timeout has been defined
-	int timeout = config->timeout == -1 ? 6000 : config->timeout;
 	pop->setTimeout(timeout);
-
 	pop->adjustSize();
 	pop->show(QPoint(screen.left() + screen.width()/2 - pop->width()/2  , m_nextPosition));
 	m_nextPosition+=pop->height();
@@ -201,6 +221,12 @@ void NotifyByPopup::update(int id, KNotifyConfig * config)
 		sendNotificationDBus(id, id, config);
 		return;
 	}
+
+	// otherwise, just display a new Growl notification
+	if(NotifyByPopupGrowl::canPopup())
+	{
+		notify( id, config );
+	}
 }
 
 void NotifyByPopup::fillPopup(KPassivePopup *pop,int id,KNotifyConfig * config)
@@ -211,7 +237,7 @@ void NotifyByPopup::fillPopup(KPassivePopup *pop,int id,KNotifyConfig * config)
 	KIconLoader iconLoader(iconName);
 	QPixmap appIcon = iconLoader.loadIcon( iconName, KIconLoader::Small );
 
-	KVBox *vb = pop->standardView( appCaption , config->image.isNull() ? config->text : QString() , appIcon );
+	KVBox *vb = pop->standardView( config->title.isEmpty() ? appCaption : config->title , config->image.isNull() ? config->text : QString() , appIcon );
 	KVBox *vb2 = vb;
 
 	if(!config->image.isNull())
@@ -453,9 +479,13 @@ void NotifyByPopup::closeNotificationDBus(int id)
 QStringList NotifyByPopup::popupServerCapabilities()
 {
 	if (!m_dbusServiceExists) {
-		// Return capabilities of the KPassivePopup implementation
-		return QStringList() << "actions" << "body" << "body-hyperlinks"
-		                     << "body-markup" << "icon-static";
+		if( NotifyByPopupGrowl::canPopup() ) {
+			return NotifyByPopupGrowl::capabilities();
+		} else {
+			// Return capabilities of the KPassivePopup implementation
+			return QStringList() << "actions" << "body" << "body-hyperlinks"
+			                     << "body-markup" << "icon-static";
+		}
 	}
 
 	if(m_dbusServiceCapCacheDirty) {
