@@ -27,9 +27,13 @@
 #include <klocale.h>
 #include <kurl.h>
 #include <kglobal.h>
+#include <KDE/KPluginFactory>
 
 #include <QtDBus/QtDBus>
 
+#define QL1S(x)  QLatin1String(x)
+#define QL1C(x)  QLatin1Char(x)
+
 /**
  * IMPORTANT: If you change anything here, please run the regression test
  * ../tests/kurifiltertest
@@ -56,40 +60,111 @@ void KAutoWebSearch::configure()
   KURISearchFilterEngine::self()->loadConfig();
 }
 
-bool KAutoWebSearch::filterUri( KUriFilterData &data ) const
+void KAutoWebSearch::populateProvidersList(QList<KUriFilterSearchProvider*>& searchProviders,
+                                           const KUriFilterData& data, bool allproviders) const
 {
-  kDebug(7023) << data.typedString();
+  QList<SearchProvider*> providers;
+  KURISearchFilterEngine *filter = KURISearchFilterEngine::self();
+  const QString searchTerm = filter->keywordDelimiter() + data.typedString();
 
-  if ( data.uriType() == KUriFilterData::Unknown && data.uri().pass().isEmpty() )
+  if (allproviders)
+    providers = SearchProvider::findAll();
+  else
   {
-    KURISearchFilterEngine *filter = KURISearchFilterEngine::self();
+    // Start with the search engines marked as preferred...
     QStringList favEngines = filter->favoriteEngineList();
     if (favEngines.isEmpty())
       favEngines = data.alternateSearchProviders();
 
-    SearchProvider *provider = filter->autoWebSearchQuery( data.typedString(), data.alternateDefaultSearchProvider() );
+    // Get rid of duplicates...
+    favEngines.removeDuplicates();
+    
+    // Sort the items...
+    qStableSort(favEngines);
 
+    // Add the search engine set as the default provider...
+    const QString defaultEngine = filter->defaultSearchEngine();
+    if (!defaultEngine.isEmpty()) {
+        favEngines.removeAll(defaultEngine);
+        favEngines.insert(0, defaultEngine);
+    }
+
+    QStringListIterator it (favEngines);
+    while (it.hasNext())
+    {
+      SearchProvider *favProvider = SearchProvider::findByDesktopName(it.next());
+      if (favProvider)
+          providers << favProvider;
+    }
+  }
+
+  for (int i = 0, count = providers.count(); i < count; ++i)
+  {
+      SearchProvider* provider = providers[i];
+      provider->setIconName(iconNameFor(provider->query(), KUriFilterData::NetProtocol));
+      searchProviders << provider;
+  }
+}
+
+bool KAutoWebSearch::filterUri( KUriFilterData &data ) const
+{
+  kDebug(7023) << data.typedString();
+
+  KUriFilterData::SearchFilterOptions option = data.searchFilteringOptions();
+
+  // Handle the flag to retrieve only preferred providers, no filtering...
+  if (option & KUriFilterData::RetrievePreferredSearchProvidersOnly)
+  {
+    QList<KUriFilterSearchProvider*> searchProviders;
+    populateProvidersList(searchProviders, data);
+    if (searchProviders.isEmpty())
+    {
+      if (!(option & KUriFilterData::RetrieveSearchProvidersOnly))
+      {
+        setUriType(data, KUriFilterData::Error);
+        setErrorMsg(data, i18n("No preferred search providers were found."));
+        return false;
+      }
+    }
+    else
+    {
+      setSearchProvider(data, QString(), data.typedString(), QL1C(KURISearchFilterEngine::self()->keywordDelimiter()));
+      setSearchProviders(data, searchProviders);
+      return true;
+    }
+  }
+
+  if (option & KUriFilterData::RetrieveSearchProvidersOnly)
+  {
+    QList<KUriFilterSearchProvider*> searchProviders;
+    populateProvidersList(searchProviders, data, true);
+    if (searchProviders.isEmpty())
+    {
+      setUriType(data, KUriFilterData::Error);
+      setErrorMsg(data, i18n("No search providers were found."));
+      return false;
+    }
+
+    setSearchProvider(data, QString(), data.typedString(), QL1C(KURISearchFilterEngine::self()->keywordDelimiter()));
+    setSearchProviders(data, searchProviders);
+    return true;
+  }
+
+  if ( data.uriType() == KUriFilterData::Unknown && data.uri().pass().isEmpty() )
+  {
+    KURISearchFilterEngine *filter = KURISearchFilterEngine::self();
+    SearchProvider *provider = filter->autoWebSearchQuery( data.typedString(), data.alternateDefaultSearchProvider() );
     if( provider )
     {
       const QString result = filter->formatResult(provider->query(), provider->charset(),
                                                   QString(), data.typedString(), true);
-      kDebug(7203) << "filtered to" << result;
       setFilteredUri( data, KUrl( result ) );
       setUriType( data, KUriFilterData::NetProtocol );
-      setSearchProvider(data, provider->name(), data.typedString(), QLatin1Char(filter->keywordDelimiter()));
+      setSearchProvider(data, provider->name(), data.typedString(), QL1C(filter->keywordDelimiter()));
 
-      KUriFilterPlugin::ProviderInfoList providerInfo;
-      Q_FOREACH(const QString &engine, favEngines)
-      {
-        SearchProvider *favProvider = SearchProvider::findByDesktopName(engine);
-        if (favProvider)
-        {
-          const QString query = favProvider->keys().at(0) + filter->keywordDelimiter() + data.typedString();
-          providerInfo.insert(favProvider->name(), qMakePair(query, iconNameFor(favProvider->query(), KUriFilterData::NetProtocol)));
-          delete favProvider;
-        }
-      }
-      setPreferredSearchProviders( data, providerInfo);
+      QList<KUriFilterSearchProvider*> searchProviders;
+      populateProvidersList(searchProviders, data);
+      setSearchProviders(data, searchProviders);
       delete provider;
       return true;
     }
