@@ -24,13 +24,16 @@
 #include <QtCore/QEventLoop>
 #include <QtCore/QTimer>
 
+#include <QtDBus/QDBusServiceWatcher>
+#include <QtDBus/QDBusPendingReply>
+
 #include <KStandardDirs>
 #include <KConfigGroup>
 #include <KDebug>
 
 
 namespace {
-    QString dbusServiceName( const QString& serviceName ) {
+    inline QString dbusServiceName( const QString& serviceName ) {
         return QString("org.kde.nepomuk.services.%1").arg(serviceName);
     }
 }
@@ -42,7 +45,9 @@ public:
     Private()
         : processControl( 0 ),
           serviceControlInterface( 0 ),
+          dbusServiceWatcher( 0 ),
           attached( false ),
+          started( false ),
           initialized( false ),
           failedToInitialize( false ) {
     }
@@ -54,11 +59,15 @@ public:
 
     ProcessControl* processControl;
     OrgKdeNepomukServiceControlInterface* serviceControlInterface;
+    QDBusServiceWatcher* dbusServiceWatcher;
 
     // true if we attached to an already running instance instead of
     // starting our own (in that case processControl will be 0)
     bool attached;
 
+    // true if we were asked to start the service
+    bool started;
+
     bool initialized;
     bool failedToInitialize;
 
@@ -66,6 +75,7 @@ public:
     QList<QEventLoop*> loops;
 
     void init( KService::Ptr service );
+    void reset();
 };
 
 
@@ -86,11 +96,31 @@ void Nepomuk::ServiceController::Private::init( KService::Ptr s )
 }
 
 
+void Nepomuk::ServiceController::Private::reset()
+{
+    initialized = false;
+    attached = false;
+    started = false;
+    failedToInitialize = false;
+    delete serviceControlInterface;
+    serviceControlInterface = 0;
+}
+
+
 Nepomuk::ServiceController::ServiceController( KService::Ptr service, QObject* parent )
     : QObject( parent ),
       d(new Private())
 {
     d->init( service );
+
+    d->dbusServiceWatcher = new QDBusServiceWatcher( dbusServiceName(name()),
+                                                     QDBusConnection::sessionBus(),
+                                                     QDBusServiceWatcher::WatchForRegistration | QDBusServiceWatcher::WatchForUnregistration,
+                                                     this );
+    connect( d->dbusServiceWatcher, SIGNAL( serviceRegistered( QString ) ),
+             this, SLOT( slotServiceRegistered( QString ) ) );
+    connect( d->dbusServiceWatcher, SIGNAL( serviceUnregistered( QString ) ),
+             this, SLOT( slotServiceUnregistered( QString ) ) );
 }
 
 
@@ -155,11 +185,11 @@ bool Nepomuk::ServiceController::start()
         return true;
     }
 
-    d->initialized = false;
-    d->failedToInitialize = false;
+    d->reset();
+    d->started = true;
 
     // check if the service is already running, ie. has been started by someone else or by a crashed instance of the server
-    // we cannot rely on the auto-restart feature of ProcessControl here. So we handle that completely in slotServiceOwnerChanged
+    // we cannot rely on the auto-restart feature of ProcessControl here. So we handle that completely via DBus signals
     if( QDBusConnection::sessionBus().interface()->isServiceRegistered( dbusServiceName( name() ) ) ) {
         kDebug() << "Attaching to already running service" << name();
         d->attached = true;
@@ -175,14 +205,10 @@ bool Nepomuk::ServiceController::start()
                      this, SLOT( slotProcessFinished( bool ) ) );
         }
 
-        connect( QDBusConnection::sessionBus().interface(),
-                 SIGNAL( serviceOwnerChanged( const QString&, const QString&, const QString& ) ),
-                 this,
-                 SLOT( slotServiceOwnerChanged( const QString&, const QString&, const QString& ) ) );
-
-        d->processControl->setCrashPolicy( ProcessControl::RestartOnCrash );
+        // we wait for the service to be registered with DBus before creating the service interface
         return d->processControl->start( KGlobal::dirs()->locate( "exe", "nepomukservicestub" ),
-                                         QStringList() << name() );
+                                         QStringList() << name(),
+                                         ProcessControl::RestartOnCrash );
     }
 }
 
@@ -193,6 +219,7 @@ void Nepomuk::ServiceController::stop()
         kDebug() << "Stopping" << name();
 
         d->attached = false;
+        d->started = false;
 
         if( d->processControl ) {
             d->processControl->setCrashPolicy( ProcessControl::StopOnCrash );
@@ -254,33 +281,40 @@ bool Nepomuk::ServiceController::waitForInitialized( int timeout )
 void Nepomuk::ServiceController::slotProcessFinished( bool /*clean*/ )
 {
     kDebug() << "Service" << name() << "went down";
-    d->initialized = false;
-    d->attached = false;
-    disconnect( QDBusConnection::sessionBus().interface() );
-    delete d->serviceControlInterface;
-    d->serviceControlInterface = 0;
+    d->reset();
     foreach( QEventLoop* loop, d->loops ) {
         loop->exit();
     }
 }
 
 
-void Nepomuk::ServiceController::slotServiceOwnerChanged( const QString& serviceName,
-                                                          const QString& oldOwner,
-                                                          const QString& newOwner )
+void Nepomuk::ServiceController::slotServiceRegistered( const QString& serviceName )
 {
-    if( !newOwner.isEmpty() && serviceName == dbusServiceName( name() ) ) {
+    if( serviceName == dbusServiceName( name() ) ) {
+        kDebug() << serviceName;
         createServiceControlInterface();
+        if( !d->processControl || !d->processControl->isRunning() ) {
+            d->attached = true;
+        }
     }
+}
 
+void Nepomuk::ServiceController::slotServiceUnregistered( const QString& serviceName )
+{
     // an attached service was not started through ProcessControl. Thus, we cannot rely
     // on its restart-on-crash feature and have to do it manually. Afterwards it is back
-    // to normals
-    else if( d->attached &&
-             oldOwner == dbusServiceName( name() ) ) {
-        kDebug() << "Attached service" << name() << "went down. Restarting ourselves.";
-        d->attached = false;
-        start();
+    // to normal
+    if( serviceName == dbusServiceName( name() ) ) {
+        if( d->attached && d->started ) {
+            kDebug() << "Attached service" << name() << "went down. Restarting ourselves.";
+            start();
+        }
+        else {
+            d->reset();
+            foreach( QEventLoop* loop, d->loops ) {
+                loop->exit();
+            }
+        }
     }
 }
 
@@ -288,16 +322,26 @@ void Nepomuk::ServiceController::slotServiceOwnerChanged( const QString& service
 void Nepomuk::ServiceController::createServiceControlInterface()
 {
     delete d->serviceControlInterface;
-
     d->serviceControlInterface = new OrgKdeNepomukServiceControlInterface( dbusServiceName( name() ),
-                                                                           "/servicecontrol",
+                                                                           QLatin1String("/servicecontrol"),
                                                                            QDBusConnection::sessionBus(),
-                                                                           this );
-    connect( d->serviceControlInterface, SIGNAL( serviceInitialized( bool ) ),
-             this, SLOT( slotServiceInitialized( bool ) ) );
-
-    if ( d->serviceControlInterface->isInitialized() ) {
-        slotServiceInitialized( true );
+                                                                          this );
+    QDBusPendingReply<bool> isInitializedReply = d->serviceControlInterface->isInitialized();
+    isInitializedReply.waitForFinished();
+    if( isInitializedReply.isError() ) {
+        kDebug() << "Failed to check service init state for" << name() << "Retrying.";
+        QMetaObject::invokeMethod( this, "createServiceControlInterface", Qt::QueuedConnection );
+    }
+    else {
+        if( isInitializedReply.value() ) {
+            slotServiceInitialized( true );
+        }
+        else {
+            kDebug() << "Service" << name() << "not initialized yet. Listening for signal.";
+            connect( d->serviceControlInterface, SIGNAL( serviceInitialized( bool ) ),
+                    this, SLOT( slotServiceInitialized( bool ) ),
+                    Qt::QueuedConnection );
+        }
     }
 }
 
