@@ -20,7 +20,6 @@
 */
 
 #include "invalidfileresourcecleaner.h"
-#include "nie.h"
 
 #include <QtCore/QList>
 #include <QtCore/QFile>
@@ -30,11 +29,9 @@
 #include <Soprano/QueryResultIterator>
 
 #include <Nepomuk/ResourceManager>
+#include <Nepomuk/Vocabulary/NIE>
 
 #include <KDebug>
-#include <KConfig>
-#include <KConfigGroup>
-#include <KUrl>
 
 
 Nepomuk::InvalidFileResourceCleaner::InvalidFileResourceCleaner( QObject* parent )
@@ -52,25 +49,6 @@ Nepomuk::InvalidFileResourceCleaner::~InvalidFileResourceCleaner()
     wait();
 }
 
-namespace {
-
-    /* BUG: This is a workaround to "Removable Storage" or "Network Storage" bug, where the metadata of all
-     * the files in the NAS would be deleted if the NAS was unmounted and Nepomuk was restarted.
-     *
-     * FIXME: This is NOT a permanent fix and should be removed in 4.7
-     */ 
-    QString deletionBlacklistFilter() {
-        KConfig config("nepomukserverrc");
-        QStringList directories = config.group("Service-nepomukfilewatch").readPathEntry("Deletion Blacklist", QStringList());
-        
-        QString filter;
-        Q_FOREACH( const QString & dir, directories ) {
-            filter += QString::fromLatin1( " && !regex(str(?url), '^%1') " )
-                      .arg( KUrl(dir).url( KUrl::AddTrailingSlash ) );
-        }
-        return filter;
-    }
-}
 
 void Nepomuk::InvalidFileResourceCleaner::run()
 {
@@ -82,9 +60,8 @@ void Nepomuk::InvalidFileResourceCleaner::run()
     QList<Soprano::Node> resourcesToRemove;
     QString query = QString::fromLatin1( "select distinct ?r ?url where { "
                                          "?r %1 ?url. "
-                                         "FILTER(regex(str(?url), 'file://') %2 ). }" )
-                    .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ),
-                          deletionBlacklistFilter() );
+                                         "FILTER(regex(str(?url), 'file://')) . }" )
+                    .arg( Soprano::Node::resourceToN3( Nepomuk::Vocabulary::NIE::url() ) );
     Soprano::QueryResultIterator it = Nepomuk::ResourceManager::instance()->mainModel()->executeQuery( query, Soprano::Query::QueryLanguageSparql );
 
     while( it.next() && !m_stopped ) {
