@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2008-2009 Sebastian Trueg <trueg@kde.org>
+   Copyright (c) 2008-2010 Sebastian Trueg <trueg@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,34 +19,51 @@
 #include "folder.h"
 #include "folderconnection.h"
 #include "queryservice.h"
-
-#include <Soprano/Model>
-#include <Soprano/Node>
+#include "countqueryrunnable.h"
+#include "searchrunnable.h"
 
 #include <Nepomuk/Resource>
 #include <Nepomuk/ResourceManager>
 
+#include <Soprano/Model>
+
 #include <KDebug>
 
+#include <QtCore/QThreadPool>
+
+
+Nepomuk::Query::Folder::Folder( const Query& query, QObject* parent )
+    : QObject( parent ),
+      m_isSparqlQueryFolder( false ),
+      m_query( query ),
+      m_currentSearchRunnable( 0 ),
+      m_currentCountQueryRunnable( 0 )
+{
+    init();
+}
+
 
 Nepomuk::Query::Folder::Folder( const QString& query, const RequestPropertyMap& requestProps, QObject* parent )
     : QObject( parent ),
-      m_query( query ),
+      m_isSparqlQueryFolder( true ),
+      m_sparqlQuery( query ),
       m_requestProperties( requestProps ),
-      m_initialListingDone( false ),
-      m_storageChanged( false )
+      m_currentSearchRunnable( 0 ),
+      m_currentCountQueryRunnable( 0 )
 {
+    init();
+}
+
+
+void Nepomuk::Query::Folder::init()
+{
+    m_resultCount = -1;
+    m_initialListingDone = false;
+    m_storageChanged = false;
+
     m_updateTimer.setSingleShot( true );
     m_updateTimer.setInterval( 2000 );
 
-    m_searchCore = new SearchCore( this );
-
-    connect( m_searchCore, SIGNAL( newResult( const Nepomuk::Query::Result& ) ),
-             this, SLOT( slotSearchNewResult( const Nepomuk::Query::Result& ) ) );
-    connect( m_searchCore, SIGNAL( scoreChanged( const Nepomuk::Query::Result& ) ),
-             this, SLOT( slotSearchScoreChanged( const Nepomuk::Query::Result& ) ) );
-    connect( m_searchCore, SIGNAL( finished() ),
-             this, SLOT( slotSearchFinished() ) );
     connect( ResourceManager::instance()->mainModel(), SIGNAL( statementsAdded() ),
              this, SLOT( slotStorageChanged() ) );
     connect( ResourceManager::instance()->mainModel(), SIGNAL( statementsRemoved() ),
@@ -58,21 +75,38 @@ Nepomuk::Query::Folder::Folder( const QString& query, const RequestPropertyMap&
 
 Nepomuk::Query::Folder::~Folder()
 {
+    if( m_currentSearchRunnable )
+        m_currentSearchRunnable->cancel();
+    if( m_currentCountQueryRunnable )
+        m_currentCountQueryRunnable->cancel();
+
+    // cannot use qDeleteAll since deleting a connection changes m_connections
+    while ( !m_connections.isEmpty() )
+        delete m_connections.first();
 }
 
 
 void Nepomuk::Query::Folder::update()
 {
-    if ( !m_searchCore->isActive() ) {
-        // run the search and forward signals to all connections that requested it
-        m_searchCore->query( m_query, m_requestProperties );
+    if ( !m_currentSearchRunnable ) {
+        m_currentSearchRunnable = new SearchRunnable( this );
+        QueryService::searchThreadPool()->start( m_currentSearchRunnable, 1 );
+
+        // we only need the count for initialListingDone
+        // count with a limit is pointless since Virtuoso will ignore the limit
+        if ( !m_initialListingDone &&
+             !m_isSparqlQueryFolder &&
+             m_query.limit() == 0 ) {
+            m_currentCountQueryRunnable = new CountQueryRunnable( this );
+            QueryService::searchThreadPool()->start( m_currentCountQueryRunnable, 0 );
+        }
     }
 }
 
 
 QList<Nepomuk::Query::Result> Nepomuk::Query::Folder::entries() const
 {
-    return m_results.values();
+    return m_results.toList();
 }
 
 
@@ -82,39 +116,56 @@ bool Nepomuk::Query::Folder::initialListingDone() const
 }
 
 
-void Nepomuk::Query::Folder::slotSearchNewResult( const Nepomuk::Query::Result& result )
+QString Nepomuk::Query::Folder::sparqlQuery() const
 {
-    if ( m_initialListingDone ) {
-        m_newResults.insert( result.resource().resourceUri(), result );
-        if ( !m_results.contains( result.resource().resourceUri() ) ) {
-            emit newEntries( QList<Result>() << result );
+    if ( !m_isSparqlQueryFolder )
+        return m_query.toSparqlQuery();
+    else
+        return m_sparqlQuery;
+}
+
+
+Nepomuk::Query::RequestPropertyMap Nepomuk::Query::Folder::requestPropertyMap() const
+{
+    if ( !m_isSparqlQueryFolder )
+        return m_query.requestPropertyMap();
+    else
+        return m_requestProperties;
+}
+
+
+// called from SearchRunnable in the search thread
+void Nepomuk::Query::Folder::addResults( const QList<Nepomuk::Query::Result>& results )
+{
+    QSet<Result> newResults;
+    Q_FOREACH( const Result& result, results ) {
+        if ( !m_results.contains( result ) ) {
+            newResults.insert( result );
         }
     }
-    else {
-        m_results.insert( result.resource().resourceUri(), result );
-        emit newEntries( QList<Result>() << result );
-    }
-}
 
-
-void Nepomuk::Query::Folder::slotSearchScoreChanged( const Nepomuk::Query::Result& )
-{
-    // TODO: implement this
     if ( m_initialListingDone ) {
-
+        m_newResults += newResults;
     }
     else {
+        m_results += newResults;
+    }
 
+    if( !newResults.isEmpty() ) {
+        emit newEntries( newResults.toList() );
     }
 }
 
 
-void Nepomuk::Query::Folder::slotSearchFinished()
+// called from SearchRunnable in the search thread
+void Nepomuk::Query::Folder::listingFinished()
 {
+    m_currentSearchRunnable = 0;
+
     if ( m_initialListingDone ) {
         // inform about removed items
         foreach( const Result& result, m_results ) {
-            if ( !m_newResults.contains( result.resource().resourceUri() ) ) {
+            if ( !m_newResults.contains( result ) ) {
                 emit entriesRemoved( QList<QUrl>() << result.resource().resourceUri() );
             }
         }
@@ -124,18 +175,21 @@ void Nepomuk::Query::Folder::slotSearchFinished()
         m_newResults.clear();
     }
     else {
+        kDebug() << "Listing done. Total:" << m_results.count();
         m_initialListingDone = true;
         emit finishedListing();
     }
 
     // make sure we do not update again right away
-    m_updateTimer.start();
+    // but we need to do it from the main thread but this
+    // method is called sync from the SearchRunnable
+    QMetaObject::invokeMethod( &m_updateTimer, "start", Qt::QueuedConnection );
 }
 
 
 void Nepomuk::Query::Folder::slotStorageChanged()
 {
-    if ( !m_updateTimer.isActive() && !m_searchCore->isActive() ) {
+    if ( !m_updateTimer.isActive() && !m_currentSearchRunnable ) {
         update();
     }
     else {
@@ -147,13 +201,25 @@ void Nepomuk::Query::Folder::slotStorageChanged()
 // if there was a change in the nepomuk store we update
 void Nepomuk::Query::Folder::slotUpdateTimeout()
 {
-    if ( m_storageChanged && !m_searchCore->isActive() ) {
+    if ( m_storageChanged && !m_currentSearchRunnable ) {
         m_storageChanged = false;
         update();
     }
 }
 
 
+// called from CountQueryRunnable in the search thread
+void Nepomuk::Query::Folder::countQueryFinished( int count )
+{
+    m_currentCountQueryRunnable = 0;
+
+    m_resultCount = count;
+    kDebug() << m_resultCount;
+    if( count >= 0 )
+        emit resultCount( m_resultCount );
+}
+
+
 void Nepomuk::Query::Folder::addConnection( FolderConnection* conn )
 {
     Q_ASSERT( conn != 0 );
@@ -172,6 +238,7 @@ void Nepomuk::Query::Folder::removeConnection( FolderConnection* conn )
 
     if ( m_connections.isEmpty() ) {
         kDebug() << "Folder unused. Deleting.";
+        emit aboutToBeDeleted( this );
         deleteLater();
     }
 }
@@ -182,4 +249,11 @@ QList<Nepomuk::Query::FolderConnection*> Nepomuk::Query::Folder::openConnections
     return m_connections;
 }
 
+
+uint Nepomuk::Query::qHash( const Result& result )
+{
+    // we only use this to ensure that we do not emit duplicates
+    return qHash(result.resource().resourceUri());
+}
+
 #include "folder.moc"
