@@ -1,5 +1,5 @@
 /* This file is part of the KDE Project
-   Copyright (c) 2007 Sebastian Trueg <trueg@kde.org>
+   Copyright (c) 2007-2010 Sebastian Trueg <trueg@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -18,6 +18,7 @@
 
 #include "nepomukcore.h"
 #include "repository.h"
+#include "ontologyloader.h"
 
 #include <KDebug>
 #include <KSharedConfig>
@@ -32,7 +33,8 @@ static const char s_repositoryName[] = "main";
 
 Nepomuk::Core::Core( QObject* parent )
     : Soprano::Server::ServerCore( parent ),
-      m_repository( 0 )
+      m_repository( 0 ),
+      m_initialized( false )
 {
     // we give the Virtuoso server a server thread max of 100 which is already an insane number
     // just make sure we never reach that limit
@@ -57,13 +59,39 @@ void Nepomuk::Core::init()
 
 bool Nepomuk::Core::initialized() const
 {
-    return( m_repository && m_repository->state() == Repository::OPEN );
+    return m_initialized;
 }
 
 
-void Nepomuk::Core::slotRepositoryOpened( Repository*, bool success )
+void Nepomuk::Core::slotRepositoryOpened( Repository* repo, bool success )
 {
-    emit initializationDone( success );
+    if( !success ) {
+        emit initializationDone( success );
+    }
+    else {
+        // create the ontology loader, let it update all the ontologies,
+        // and only then mark the service as initialized
+        // TODO: fail the initialization in case loading the ontologies
+        // failed.
+        OntologyLoader* ontologyLoader = new OntologyLoader( repo, this );
+        connect( ontologyLoader, SIGNAL(ontologyLoadingFinished(Nepomuk::OntologyLoader*)),
+                 this, SLOT(slotOntologiesLoaded()) );
+        ontologyLoader->updateLocalOntologies();
+    }
+}
+
+
+void Nepomuk::Core::slotOntologiesLoaded()
+{
+    // once ontologies are updated we should update the query prefixes
+    m_repository->setEnableQueryPrefixExpansion(false);
+    m_repository->setEnableQueryPrefixExpansion(true);
+
+    if ( !m_initialized ) {
+        // and finally we are done: the repository is online and the ontologies are loaded.
+        m_initialized = true;
+        emit initializationDone( true );
+    }
 }
 
 
