@@ -1,5 +1,6 @@
 /* This file is part of the KDE Project
    Copyright (c) 2008 Sebastian Trueg <trueg@kde.org>
+   Copyright (c) 2010 Vishesh Handa <handa.vish@gmail.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -43,27 +44,25 @@ namespace {
 Nepomuk::EventMonitor::EventMonitor( IndexScheduler* scheduler, QObject* parent )
     : QObject( parent ),
       m_indexScheduler( scheduler ),
-      m_pauseState( NotPaused )
+      m_pauseState( NotPaused ),
+      m_totalIndexingSeconds( 0 )
 {
-    // FileSystemWatcher does not catch changes to files, only new and removed files
-    // thus, we also do periodic updates of the whole index every two hours
-    connect( &m_periodicUpdateTimer, SIGNAL( timeout() ),
-             m_indexScheduler, SLOT( updateAll() ) );
-    m_periodicUpdateTimer.setInterval( 2*60*60*1000 );
-
     // monitor the powermanagement to not drain the battery
     connect( Solid::PowerManagement::notifier(), SIGNAL( appShouldConserveResourcesChanged( bool ) ),
              this, SLOT( slotPowerManagementStatusChanged( bool ) ) );
 
+    // monitor the index scheduler since there is no need for certain checks when it is not indexing
+    connect( m_indexScheduler, SIGNAL(indexingStateChanged(bool)),
+             this, SLOT(slotIndexingStateChanged(bool)) );
+
     // setup the avail disk usage monitor
     connect( &m_availSpaceTimer, SIGNAL( timeout() ),
              this, SLOT( slotCheckAvailableSpace() ) );
-    m_availSpaceTimer.start( 20*1000 ); // every 20 seconds should be enough
 
     if ( StrigiServiceConfig::self()->isInitialRun() ) {
         // TODO: add actions to this notification
 
-        m_initialIndexTime.start();
+        m_indexingStartTime = QDateTime::currentDateTime();
 
         // inform the user about the initial indexing
         sendEvent( "initialIndexingStarted",
@@ -76,9 +75,9 @@ Nepomuk::EventMonitor::EventMonitor( IndexScheduler* scheduler, QObject* parent
         connect( m_indexScheduler, SIGNAL( indexingStopped() ),
                  this, SLOT( slotIndexingStopped() ),
                  Qt::QueuedConnection );
-    }
-    else {
-        m_periodicUpdateTimer.start();
+                 
+        connect( m_indexScheduler, SIGNAL( indexingSuspended(bool) ),
+                 this, SLOT( slotIndexingSuspended(bool) ) );
     }
 
     slotPowerManagementStatusChanged( Solid::PowerManagement::appShouldConserveResources() );
@@ -94,16 +93,14 @@ void Nepomuk::EventMonitor::slotPowerManagementStatusChanged( bool conserveResou
 {
     if ( !conserveResources && m_pauseState == PausedDueToPowerManagement ) {
         kDebug() << "Resuming indexer due to power management";
-        m_pauseState = NotPaused;
-        m_indexScheduler->resume();
+        resumeIndexing();
         sendEvent( "indexingResumed", i18n("Resuming indexing of files for fast searching."), "battery-charging" );
     }
     else if ( conserveResources &&
               m_indexScheduler->isRunning() &&
               !m_indexScheduler->isSuspended() ) {
         kDebug() << "Pausing indexer due to power management";
-        m_pauseState = PausedDueToPowerManagement;
-        m_indexScheduler->suspend();
+        pauseIndexing( PausedDueToPowerManagement );
         sendEvent( "indexingSuspended", i18n("Suspending the indexing of files to preserve resources."), "battery-100" );
     }
 }
@@ -116,8 +113,7 @@ void Nepomuk::EventMonitor::slotCheckAvailableSpace()
         if ( info.available() <= StrigiServiceConfig::self()->minDiskSpace() ) {
             if ( m_indexScheduler->isRunning() &&
                 !m_indexScheduler->isSuspended() ) {
-                m_pauseState = PausedDueToAvailSpace;
-                m_indexScheduler->suspend();
+                pauseIndexing( PausedDueToAvailSpace );
                 sendEvent( "indexingSuspended",
                            i18n("Disk space is running low (%1 left). Suspending indexing of files.",
                                 KIO::convertSize( info.available() ) ),
@@ -126,8 +122,7 @@ void Nepomuk::EventMonitor::slotCheckAvailableSpace()
         }
         else if ( m_pauseState == PausedDueToAvailSpace ) {
             kDebug() << "Resuming indexer due to disk space";
-            m_pauseState = NotPaused;
-            m_indexScheduler->resume();
+            resumeIndexing();
             sendEvent( "indexingResumed", i18n("Resuming indexing of files for fast searching."), "drive-harddisk" );
         }
     }
@@ -142,15 +137,63 @@ void Nepomuk::EventMonitor::slotIndexingStopped()
 {
     // inform the user about the end of initial indexing. This will only be called once
     if ( !m_indexScheduler->isSuspended() ) {
-        kDebug() << "initial indexing took" << m_initialIndexTime.elapsed();
+        m_totalIndexingSeconds += m_indexingStartTime.secsTo( QDateTime::currentDateTime() );
+        const int elapsed = m_totalIndexingSeconds * 1000;
+        
+        kDebug() << "initial indexing took" << elapsed;
         sendEvent( "initialIndexingFinished",
-                   i18nc( "@info %1 is a duration formatted using KLocale::formatDuration",
+                   i18nc( "@info %1 is a duration formatted using KLocale::prettyFormatDuration",
                           "Initial indexing of files for fast searching finished in %1",
-                          KGlobal::locale()->formatDuration( m_initialIndexTime.elapsed() ) ),
+                          KGlobal::locale()->prettyFormatDuration( elapsed ) ),
                    "nepomuk" );
         m_indexScheduler->disconnect( this );
+    }
+}
 
-        m_periodicUpdateTimer.start();
+
+void Nepomuk::EventMonitor::pauseIndexing(int pauseState)
+{
+    m_pauseState = pauseState;
+    m_indexScheduler->suspend();
+
+    m_totalIndexingSeconds += m_indexingStartTime.secsTo( QDateTime::currentDateTime() );
+}
+
+
+void Nepomuk::EventMonitor::resumeIndexing()
+{
+    m_pauseState = NotPaused;
+    m_indexScheduler->resume();
+
+    m_indexingStartTime = QDateTime::currentDateTime();
+}
+
+
+void Nepomuk::EventMonitor::slotIndexingSuspended( bool suspended )
+{
+    if( suspended ) {
+        //The indexing is already paused, this meerly sets the correct state, and adjusts the timing.
+        pauseIndexing( PausedCustom );
+    }
+    else {
+        //Again, used to set the correct state, and adjust the timing.
+        resumeIndexing();
+    }
+}
+
+
+void Nepomuk::EventMonitor::slotIndexingStateChanged(bool indexing)
+{
+    // there is no need to check the available space (and wasting IO) when we are not indexing
+    // the only exception is when we suspended due to disk space shortage since we can resume once
+    // disk space has been freed up
+    if( indexing ) {
+        kDebug() << "Starting available disk space timer.";
+        m_availSpaceTimer.start( 20*1000 ); // every 20 seconds should be enough
+    }
+    else if( m_pauseState != PausedDueToAvailSpace ) {
+        kDebug() << "Stopping available disk space timer.";
+        m_availSpaceTimer.stop();
     }
 }
 
