@@ -90,7 +90,20 @@ std::ostream &operator<<(std::ostream &out, const QVariant &value)
     case QVariant::ULongLong:
         out << value.toString()
             << "  (0x" << QString::number(value.toULongLong(), 16) << ")  (" << QVariant::typeToName(value.type()) << ")";
+        break;        
+    case QVariant::UserType:
+    {
+        //qDebug() << "got variant type:" << value.typeName();
+        if (value.canConvert<QList<int> >())
+        {
+            QList<int> intlist = value.value<QList<int> >();
+            QStringList tmp;
+            foreach (int val, intlist)
+                tmp.append(QString::number(val));
+            out << "{" << tmp.join(",") << "} (int list)";
+        }
         break;
+    }
     default:
         out << "'" << value.toString() << "'  (string)";
         break;
@@ -104,6 +117,7 @@ std::ostream &operator<<(std::ostream &out, const Solid::Device &device)
     out << "  parent = " << QVariant(device.parentUdi()) << endl;
     out << "  vendor = " << QVariant(device.vendor()) << endl;
     out << "  product = " << QVariant(device.product()) << endl;
+    out << "  description = " << QVariant(device.description()) << endl;
 
     int index = Solid::DeviceInterface::staticMetaObject.indexOfEnumerator("Type");
     QMetaEnum typeEnum = Solid::DeviceInterface::staticMetaObject.enumerator(index);
@@ -225,7 +239,7 @@ int main(int argc, char **argv)
       cout << "  solid-hardware unmount 'udi'" << endl;
       cout << i18n("             # If applicable, unmount the device corresponding to 'udi'.\n") << endl;
 
-      cout << "  solid-hardware hardware eject 'udi'" << endl;
+      cout << "  solid-hardware eject 'udi'" << endl;
       cout << i18n("             # If applicable, eject the device corresponding to 'udi'.\n") << endl;
 
       return 0;
